#! /bin/sh

BIN_DIR=/usr/bin
LIB_DIR=/usr/lib
UDEV_SIGNPAD_RULES=/etc/udev/rules.d/39-signpad.rules
CFG_FILE=/etc/SignPad.conf
ICA_CLIENT=/opt/Citrix/ICAClient

BIN_FILES='SignPadInit'
LIB_FILES='libspusb-1.0.so.0 spvc32.so spvchelper'

die()
{
  echo "$@"
  exit 2
}

remove_file()
{
  if [ -f "$1" ]
  then
    any=true
    rm "$1" || ok=false
  fi
}

have_udev()
{
  test -d /etc/udev/rules.d
}

usage()
{
  echo "Usage: $0 install"
  echo "       $0 uninstall"
  exit 1
}

install_udev()
{
  if ! grep '^signpad:' /etc/group >/dev/null
  then
    groupadd signpad || die 'Cannot create group `signpad'\''.'
  fi
  s=udev-signpad-3.rules
  install -m 644 $s $UDEV_SIGNPAD_RULES || die "Cannot install udev rules"
  if udevadm help >/dev/null 2>/dev/null
  then
    if udevadm control --help | fgrep reload_rules >/dev/null
    then
      udevadm control --reload_rules
    else
      udevadm control --reload-rules
    fi
  else
    udevcontrol reload_rules
  fi
  echo 'Please add users who want to use the SignPad device to the `signpad'\'' group.'
}

install_init()
{
  if [ -x /usr/sbin/update-rc.d ]
  then
    install -m 755 init-debian /etc/init.d/SignPad || exit
    /usr/sbin/update-rc.d SignPad defaults
  elif [ -x /sbin/insserv ]
  then
    install -m 755 init-suse /etc/init.d/SignPad || exit
    /sbin/insserv /etc/init.d/SignPad
  elif [ -x /sbin/rc-update ]
  then
    install -m 755 init-gentoo /etc/init.d/SignPad || exit
    /sbin/rc-update add SignPad default
  elif [ -x /sbin/chkconfig ]
  then
    install -m 755 init-redhat /etc/init.d/SignPad || exit
    /sbin/chkconfig --add SignPad --level 2345
  else
    echo 'Unsupported Linux distribution, SignPad initialization script not installed'
  fi
}

install_cfg()
{
  echo SIGNPAD_ONCE=$1 > $CFG_FILE || die "Cannot write $CFG_FILE"
  chmod 644 $CFG_FILE|| die "Cannot chmod $CFG_FILE"
}

find_module_ini()
{
  MODULE_INI=
  if ! test -d "$ICA_CLIENT"
  then
    echo "ICA Client not found, please modify module.ini manually"
    return 1
  fi
  p="$ICA_CLIENT/config/module.ini"
  if ! test -e "$p"
  then
    echo "$ICA_CLIENT/config/module.ini does not exist"
    return 1
  fi
  if test -h "$p"
  then
    x=$(readlink "$p")
    test -z "$x" && return 1
    case "$x" in
      /*) MODULE_INI="$x";;
      *)  MODULE_INI="$ICA_CLIENT/config/$x";;
    esac
  else
    MODULE_INI="$p"
  fi
  return 0
}

install_plugin()
{
  find_module_ini || return  
  x=1
  while test -e "$MODULE_INI.bak.$x"
  do
    x=$(($x+1))
  done
  mv "$MODULE_INI" "$MODULE_INI.bak.$x" || return
  cp "$MODULE_INI.bak.$x" "$MODULE_INI" || return
  if ! grep '^VirtualDriver[ 	]*=.*SPVC' "$MODULE_INI" >/dev/null
  then
    sed 's/\(^VirtualDriver[ 	]*=.*\)$/\1, SPVC/' "$MODULE_INI" > "$MODULE_INI.tmp" || return
    mv "$MODULE_INI.tmp" "$MODULE_INI" || return
  fi
  if ! grep '^SPVC[ 	]*=[ 	]*[Oo][Nn]' "$MODULE_INI" >/dev/null
  then
    sed '/^VirtualDriver[ 	]*=/a\SPVC = On' "$MODULE_INI" > "$MODULE_INI.tmp" || return
    mv "$MODULE_INI.tmp" "$MODULE_INI" || return
  fi
  if ! grep '^\[SPVC\]' "$MODULE_INI" >/dev/null
  then
    echo '' >> "$MODULE_INI"
    echo '[SPVC]' >> "$MODULE_INI"
    echo 'DriverName=/usr/lib/spvc32.so' >> "$MODULE_INI"
    echo 'TraceLevel=1' >> "$MODULE_INI"
  fi
}

uninstall_plugin()
{
  find_module_ini || return  
  x=1
  while test -e "$MODULE_INI.bak.$x"
  do
    x=$(($x+1))
  done
  mv "$MODULE_INI" "$MODULE_INI.bak.$x" || return
  cp "$MODULE_INI.bak.$x" "$MODULE_INI" || return

  if grep '^VirtualDriver[ 	]*=.*SPVC' "$MODULE_INI" >/dev/null
  then
    sed '/^VirtualDriver[ 	]*=/s/,[ 	]SPVC//g' "$MODULE_INI" > "$MODULE_INI.tmp" || return
    mv "$MODULE_INI.tmp" "$MODULE_INI" || return
    any=true
  fi

  if grep '^SPVC[ 	]*=[ 	]*[Oo][Nn]' "$MODULE_INI" >/dev/null
  then
    sed '/SPVC[ 	]*=[ 	]*[Oo][Nn][ 	]*$/d' "$MODULE_INI" > "$MODULE_INI.tmp" || return
    mv "$MODULE_INI.tmp" "$MODULE_INI" || return
    any=true
  fi

  if grep '^\[SPVC\]' "$MODULE_INI" >/dev/null
  then
    sed -e '/^\[SPVC\]/,/^\[/d' "$MODULE_INI" > "$MODULE_INI.tmp" || return
    mv "$MODULE_INI.tmp" "$MODULE_INI" || return
    if [ -z $(tail -n 1 "$MODULE_INI") ]
    then
      head -n -1 "$MODULE_INI" > "$MODULE_INI.tmp" || return
      mv "$MODULE_INI.tmp" "$MODULE_INI" || return
    fi
    any=true
  fi
}

do_install()
{
  test $# -eq 1 || usage
  test "`id -u`" = 0 || die 'Must run as root'
  for x in $LIB_FILES
  do
    install -m 755 $x "$LIB_DIR"/ || die "Cannot install $x"
  done
  for x in $BIN_FILES
  do
    install -m 755 $x "$BIN_DIR"/ || die "Cannot install $x"
  done
  ldconfig || die "ldconfig failed"
  install_init
  if have_udev
  then
    install_udev
    install_cfg false
  else
    install_cfg true
    echo 'udev not installed, cannot automatically set permissions.'
  fi
  install_plugin
  /etc/init.d/SignPad start
}

uninstall_init()
{
  if [ -x /usr/sbin/update-rc.d ]
  then
    remove_file /etc/init.d/SignPad
    /usr/sbin/update-rc.d SignPad remove
  elif [ -x /sbin/insserv ]
  then
    /sbin/insserv -r /etc/init.d/SignPad
    remove_file /etc/init.d/SignPad
  elif [ -x /sbin/rc-update ]
  then
    remove_file /etc/init.d/SignPad
    /sbin/rc-update del SignPad
  elif [ -x /sbin/chkconfig ]
  then
    /sbin/chkconfig --del SignPad
    remove_file /etc/init.d/SignPad
  fi
}

uninstall_udev()
{
  if [ -e $UDEV_SIGNPAD_RULES ]
  then
    rm $UDEV_SIGNPAD_RULES
  fi
  if grep '^signpad:' /etc/group >/dev/null
  then
    echo 'You may want to remove some users from the `signpad'\'' group'
    echo 'and remove that group.'
  fi
}

do_uninstall()
{
  test $# -eq 1 || usage
  test "`id -u`" = 0 || die 'Must run as root'
  test -x /etc/init.d/SignPad && /etc/init.d/SignPad stop
  any=false
  ok=true
  if have_udev
  then
    uninstall_udev
  fi
  uninstall_init
  remove_file $CFG_FILE
  for x in $LIB_FILES
  do
    remove_file "$LIB_DIR/$x"
  done
  for x in $BIN_FILES
  do
    remove_file "$BIN_DIR/$x"
  done
  uninstall_plugin
  if ! $ok
  then
    echo 'Could not uninstall spvc.'
  elif $any
  then
    ldconfig
    echo 'spvc uninstalled.'
  else
    echo 'spvc not installed, nothing done.'
  fi
}

cd "`dirname $0`"
case "$1" in
  install)    do_install "$@";;
  uninstall)  do_uninstall "$@";;
  *)          usage;;
esac
